//-----------------------------------------------------------------------------
//
//  $Id: AudioStreamSelection.h 912 2015-07-26 00:50:29Z weegreenblobbie $
//
//  Nsound is a C++ library and Python module for audio synthesis featuring
//  dynamic digital filters. Nsound lets you easily shape waveforms and write
//  to disk or plot them. Nsound aims to be as powerful as Csound but easy to
//  use.
//
//  Copyright (c) 2009-Present Nick Hilton
//
//  weegreenblobbie_yahoo_com (replace '_' with '@' and '.')
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//-----------------------------------------------------------------------------
#ifndef _NSOUND_AUDIO_STREAM_SELECTION_H_
#define _NSOUND_AUDIO_STREAM_SELECTION_H_

#include <Nsound/Nsound.h>

#include <vector>

namespace Nsound
{

// Forward
class AudioStream;

typedef std::vector< std::vector< boolean > > BooleanVectorVector;

//-----------------------------------------------------------------------------
//! A helper class for advance operators.
class AudioStreamSelection
{

public:

    AudioStreamSelection(AudioStream & as, const BooleanVectorVector & bv);
    AudioStreamSelection(const AudioStreamSelection & copy);
    AudioStreamSelection & operator=(const AudioStreamSelection & rhs);
    ~AudioStreamSelection(){}

    AudioStreamSelection & operator+=(const float64 & rhs);
    AudioStreamSelection & operator-=(const float64 & rhs);
    AudioStreamSelection & operator*=(const float64 & rhs);
    AudioStreamSelection & operator/=(const float64 & rhs);
    AudioStreamSelection & operator^=(const float64 & rhs);

    AudioStreamSelection & operator=(const float64 & rhs);

    // For SWIG
    void
    set(const float64 & rhs){this->operator=(rhs);};

private:

    AudioStream * target_as_;
    BooleanVectorVector bv_;
};

};

#endif
